#Snapshots Module version 6.6
# The following functions can be utilized to run a kmcos simulation with a user
# defined snapshot size. The associated KMC data (simulation name, parameters
# of interest (user defined), atoms.tof_data, atoms.tof_integ, atoms.occupation)
# will be written to .csv files. In addition, the configuration and a
# simulation log will be written when the simulation is complete.
import time
try:
    import kmcos.snapshots_globals as sg
except:
    import snapshots_globals as sg

# This function creates the output file for the TOF data, occupations data and
# parameters file. The file that is created to hold the data is called
# simulation_name_TOFs_and_Coverages.csv where simulation_name is user defined
# in snapshots_globals.py. The parameters of interest will be printed to the
# same file, if any exist. If desired, parameter initial values will be written
# to a separate file called initial_parameters.txt. By default, the output file
# of the .csv file is created and initial_parameters.txt written, but these
# output files can be turned off by the option of True or False for
# write_output.
def create_headers(print_all_parameters=True):

    # Keeping track of the start time for runtime info
    sg.start = time.time()

    # Check the parameters of interest and write them if requested
    data_file_name = sg.simulation_name + str('_TOFs_and_Coverages.csv')
    with open(data_file_name, 'w') as simulation_data:
        simulation_data.write('Simulation_name,')
        simulation_data.write('Snapshot,')
        simulation_data.write('Step,')
        simulation_data.write('Time_(s),')
        simulation_data.write('sps,')
        simulation_data.write('tps,')
        sg.snapshot_output_headers.append('Simulation_name')
        sg.snapshot_output_headers.append('Step')
        sg.snapshot_output_headers.append('Time_(s)')
        sg.snapshot_output_headers.append('sps')
        sg.snapshot_output_headers.append('tps')
        # None and [] will lead to a 'False' evaluation, but a non-empty list
        # evaluates as 'True'
        if sg.parameters_of_interest:
            for i in range(len(sg.parameters_of_interest)):
                simulation_data.write('%s,' %(sg.parameters_of_interest[i]))
                sg.snapshot_output_headers.append(sg.parameters_of_interest[i])

        # Write the TOF_headers
        for i in range(len(sg.TOF_header_array)):
            simulation_data.write('%s_tof_data,' %(sg.TOF_header_array[i]))
            sg.snapshot_output_headers.append(sg.TOF_header_array[i] + '_tof_data')
        for i in range(len(sg.TOF_header_array)):
            simulation_data.write('%s_tof_integ,' %(sg.TOF_header_array[i]))
            sg.snapshot_output_headers.append(sg.TOF_header_array[i] + '_tof_integ')

        # Write the occupations_headers
        for i in range(len(sg.occ_header_array)):
            simulation_data.write('%s,' %(sg.occ_header_array[i]))
            sg.snapshot_output_headers.append(sg.occ_header_array[i])
        simulation_data.write('\n')

        # Write the initial TOF and coverage values
        simulation_data.write('%s,' %(sg.simulation_name))
        simulation_data.write('%s,' %(sg.snapshots_so_far))
        simulation_data.write('%s,' %(sg.steps_so_far))
        simulation_data.write('%s,' %(sg.atoms.kmc_time))
        simulation_data.write('%s,' %(0))
        simulation_data.write('%s,' %(0))
        # None and [] will lead to a 'False' evaluation, but a non-empty list
        # evaluates as 'True'
        if sg.parameters_of_interest:
            for i in range(len(sg.parameters_of_interest)):
                parameter = getattr(sg.model.parameters,
                    sg.parameters_of_interest[i])['value']
                simulation_data.write('%s,' %(parameter))

        # Write the TOF_headers
        for i in range(len(sg.TOF_header_array)):
            simulation_data.write('%s,' %(sg.atoms.tof_data[i]))
        for i in range(len(sg.TOF_header_array)):
            simulation_data.write('%s,' %(sg.atoms.tof_integ[i]))

        # Write the initial occupation values. These are in a 2-D numpy array.
        for species in range(len(sg.model.settings.species_tags)):
            for site in range(len(sg.model.settings.site_names)):
                simulation_data.write('%s,' %(sg.atoms.occupation[species][site]))
        simulation_data.write('\n')

    # Check to see if the parameters file is wanted
    if print_all_parameters:
        parameters_file_name = sg.simulation_name + '_initial_parameters.txt'
        with open(parameters_file_name, 'w') as simulation_parameters:
            # Use the parameters dictionary in KMC_settings.py to write the
            # initial values and keys
            for index, key in enumerate(sg.model.settings.parameters):
                simulation_parameters.write('%s = %s\n'
                    %(key,sg.model.settings.parameters[key]['value']))

# This function will create a nested list with all of the snapshot data. Right
# now it is not currently used for anything, but it could be used as an
# in-memory history of the last snapshot.
def save_snapshot_output_as_list():
    sg.last_snapshot_outputs = []
    sg.last_snapshot_outputs.append(sg.simulation_name)
    sg.last_snapshot_outputs.append(sg.snapshots_so_far)
    sg.last_snapshot_outputs.append(sg.steps_so_far)
    sg.last_snapshot_outputs.append(sg.atoms.kmc_time)
    sg.last_snapshot_outputs.append(sg.sps_actual)
    sg.last_snapshot_outputs.append(sg.tps_actual)
    # None and [] will lead to a 'False' evaluation, but a non-empty list
    # evaluates as 'True'
    if sg.parameters_of_interest:
        for i in range(len(sg.parameters_of_interest)):
            parameter = getattr(sg.model.parameters,
                sg.parameters_of_interest[i])['value']
            sg.last_snapshot_outputs.append(parameter)

    # Add TOF and occupation data to the last_snapshots_outputs data structure.
    # TOF data is in 1-D numpy arrays, and occupation data is in a 2-D numpy
    # array. The occupation header array is a simple 1-D list.
    for i in range(len(sg.TOF_header_array)):
        sg.last_snapshot_outputs.append(sg.atoms.tof_data[i])
    for i in range(len(sg.TOF_header_array)):
        sg.last_snapshot_outputs.append(sg.atoms.tof_integ[i])
    for species in range(len(sg.model.settings.species_tags)):
        for site in range(len(sg.model.settings.site_names)):
            sg.last_snapshot_outputs.append(sg.atoms.occupation[species][site])

# This function will write the snapshot data to disk.
def write_snapshot_data():
    data_file_name = sg.simulation_name + str('_TOFs_and_Coverages.csv')
    with open(data_file_name, 'a') as simulation_data:

        simulation_data.write('%s,' %(sg.simulation_name))
        simulation_data.write('%s,' %(sg.snapshots_so_far))
        simulation_data.write('%s,' %(sg.steps_so_far))
        simulation_data.write('%s,' %(sg.atoms.kmc_time))
        simulation_data.write('%s,' %(sg.sps_actual))
        simulation_data.write('%s,' %(sg.tps_actual))
        # None and [] will lead to a 'False' evaluation, but a non-empty list
        # evaluates as 'True'
        if sg.parameters_of_interest:
            for i in range(len(sg.parameters_of_interest)):
                parameter = getattr(sg.model.parameters,
                    sg.parameters_of_interest[i])['value']
                simulation_data.write('%s,' %(parameter))

        # Write the KMC data associated with each of the TOF_headers
        for i in range(len(sg.TOF_header_array)):
            simulation_data.write('%s,' %(sg.atoms.tof_data[i]))
        for i in range(len(sg.TOF_header_array)):
            simulation_data.write('%s,' %(sg.atoms.tof_integ[i]))

        # Write the KMC data associated with each of the
        # occupations_headers. The occupation array is 2-D, unlike the
        # header array (actually a 1-D list).
        for species in range(len(sg.model.settings.species_tags)):
            for site in range(len(sg.model.settings.site_names)):
                simulation_data.write('%s,' %(sg.atoms.occupation[species][site]))
        simulation_data.write('\n')

# This function will perform a KMC_snapshot and writes the output to the .csv
# file created in the function create_headers.
def do_snapshots(n_snapshots, sps, tps=None, acc=False):

    # sg.last_snapshot_outputs will be used to write all info (simulation name,
    # parameters of interest (user defined), atoms.tof_data, atoms.tof_integ,
    # atoms.occupation) to a list and can be used for indexing based on column
    # number.

    # This loop will do "snapshot" iterations and will run a snapshot each time.
    for snapshot in range(1, n_snapshots+1):
        if snapshot <= n_snapshots:
            sg.steps_before_snapshot = sg.steps_so_far
            if tps is None:
                if not acc:
                    sg.model.do_steps(sps)
                else:
                    sg.model.do_acc_steps(sps)
            else:
                try:
                    # If TPS is specified, try to match it with the appropriate
                    # routine
                    # Simultanoeus use of TPS and temporal acceleration is currently
                    # not supported
                    sps_actual = sg.model.do_steps_time(tps, sps)
                except:
                    # Something went wrong, probably because we don't have this
                    # custom add-on to kmcos, so just fall back to the default
                    # routine.
                    print('WARNING: TPS specified but this version of kmcos does not support time-based snapshots. Using fixed-step snapshots instead.')
                    if not acc:
                        sg.model.do_steps(sps)
                    else:
                        sg.model.do_acc_steps(sps)
            sg.atoms = sg.model.get_atoms(geometry=False)
            sg.steps_so_far = sg.atoms.kmc_step
            sg.sps_actual = sg.steps_so_far - sg.steps_before_snapshot
            sg.snapshots_so_far += 1
            sg.tps_actual = sg.atoms.kmc_time - sg.kmc_time
            sg.kmc_time = sg.atoms.kmc_time

        save_snapshot_output_as_list()

        # Check to see that the output files are requested
        if sg.write_output:
            if sg.snapshots_sampling == None:
                write_snapshot_data()
            elif sg.snapshots_so_far%sg.snapshots_sampling ==0: #This will only write output every sg.snapshots_sampling snapshots (e.g., every 10 snapshots for a value of 10).
                write_snapshot_data()
                

    # update the global lists
    sg.TOF_data_list = sg.atoms.tof_data.tolist()
    sg.TOF_integ_list = sg.atoms.tof_integ.tolist()
    sg.occ_data_list = sg.atoms.occupation.tolist()

    # Save the lattice configuration
    sg.config = sg.model._get_configuration().tolist()

    # Save the PRNG state
    try:
        sg.PRNG_state = sg.model.proclist.get_seed().tolist()
    except:
        sg.PRNG_state = None

# This function will optionally dump the configuration and create the log file
# for a simulation run. The simulation run time is calculated in this function
# based on the start time from create_headers, the log file is written to a file
# called simulation_name_log.txt and if requested, all final parameters will be
# written to a file called 'final_parameters.txt'
def create_log(dump_configuration=True, print_all_parameters=True):
    if dump_configuration:
        sg.model.dump_config(sg.simulation_name + '_final_config')
    if print_all_parameters:
        parameters_file_name = sg.simulation_name + '_final_parameters.txt'
        with open(parameters_file_name, 'w') as simulation_parameters:
            # Use the parameters dictionary in KMC_settings.py to write the
            # initial values and keys
            for index, key in enumerate(sg.model.settings.parameters):
                simulation_parameters.write('%s = %s\n'
                    %(key,sg.model.settings.parameters[key]['value']))
    stop = time.time() # stop the timer right before last exports.
    run_time = stop - sg.start
    simulation_log_name = '%s_log.txt' %(sg.simulation_name)
    with open(simulation_log_name, 'w') as simulation_log:
        simulation_log.write('total number of snapshots = %s\n'%(sg.snapshots_so_far))
        simulation_log.write('total number of steps = %s\n'%(sg.steps_so_far))
        simulation_log.write('total simulation time = %s\n' %(sg.atoms.kmc_time))
        simulation_log.write('simulation run time (wall clock time) = %s\n' %(run_time))
        # None and [] will lead to a 'False' evaluation, but a non-empty list
        # evaluates as 'True'
        if sg.parameters_of_interest:
            for i in range(len(sg.parameters_of_interest)):
                parameter = getattr(sg.model.parameters,
                    sg.parameters_of_interest[i])['value']
                simulation_log.write('%s = %s\n' %(sg.parameters_of_interest[i], parameter))

# This is a function to seed the random number generator with the requested
# state. It takes two arguments: whether the simulation is a restart or not
# and the actual PRNG state to use. In the event of a new simulation, the
# state is initialized with a single 64 bit integer; otherwise it is an array
# of 64 bit integers. For a restart, the number of integers needed is
# compiler-dependent, and there is no good way to check this a priori. It is
# incumbent upon the user to make sure that the same compiler is used for all
# segments of a multi-segment simulation.
def seed_PRNG(restart=False, state=None):

    if not restart:
        if state is None:
            print('New simulation with no random seed supplied -- using kmcos default.')
            return
        else:
            try:
                prng_state = sg.model.proclist.seed_gen(state)
                sg.model.proclist.put_seed(prng_state)
            except:
                print('PRNG seeding routines not available -- using kmcos default.')
                return

    else:
        if state is None:
            print('Restarted simulation but no PRNG state supplied -- using new default state.')
            return
        else:
            try:
                sg.model.proclist.put_seed(state)
            except:
                print('PRNG seeding routines not available -- using kmcos default.')
                return

#For snapshots module. Requires the module_state_export_import  module
def reload(sg_module_state):
    # Load module state
    sg_module_state.load_params()
    
    # Reset number of steps and time
    try:
        sg.model.base.set_kmc_step(sg.steps_so_far)
    except:
        print("Warning: sg.model.base.set_kmc_step(sg.steps_so_far) function does not exist. Proceeding without it. Sutton once wrote it. Savara may have it somewhere.")
        pass
    else:
        pass
    sg.model.base.set_kmc_time(sg.kmc_time)
    sg.atoms.kmc_step = sg.steps_so_far

    # Load the lattice
    import numpy as np
    sg.model._set_configuration(np.array(sg.config))
    sg.model._adjust_database()

    # Read the PRNG state, if available, and set it in the model if possible.
    seed_PRNG(restart=True, state=sg.PRNG_state)

    #Update the rate constants based on the temperature etc. 
    #This is necessary, or rate constants will be those of kmc_settings parameter settings.
    try:
        sg.kmcos.run.set_rate_constants() # <-- was getting a math domain error for some reason...
    except:
        print("Warning: kmcos.run.set_rate_constants() did not work correctly in the restart. \n \
               This is a known bug, but the reason is not yet looked into. \n \
               You may need to throw away your first restarted datapoint,  \n \
               and if it affects your system dynamics then this bug needs to be fixed.")



#!/usr/bin/env python3
"""A GUI frontend to create and edit kMC models.
"""
from __future__ import print_function
#    Copyright 2009-2013 Max J. Hoffmann (mjhoffmann@gmail.com)
#    This file is part of kmcos.
#
#    kmcos is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    kmcos is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with kmcos.  If not, see <http://www.gnu.org/licenses/>.

# standard modules
import optparse
import io
import sys
import os

from kmcos.types import Project, Layer, LayerList, Meta, OutputList, \
                       Parameter, Process, ProcessList, SpeciesList, \
                       OutputItem, Species
from kmcos.gui.forms import LayerEditor, MetaForm, InlineMessage, OutputForm, \
                       ParameterForm, ProcessForm, BatchProcessForm, \
                       SpeciesForm, SpeciesListForm, LatticeForm
from kmcos.config import GLADEFILE
import kmcos.io

import gobject
try:
    import pygtk
    pygtk.require('2.0')
    import gtk
except ModuleNotFoundError:
    import gi
    gi.require_version('Gtk', '3.0')
    from gi.repository import Gtk as gtk

#Kiwi imports
import kiwi.ui
from kiwi.ui.delegates import SlaveDelegate, GladeDelegate
from kiwi.ui.objectlist import ObjectTree, Column
import kiwi.ui.dialogs


menu_layout = """\
<ui>
  <menubar name="MainMenuBar">
    <menu action="MenuFile">
      <menuitem action="FileNew"/>
      <menuitem action="FileOpenProject"/>
      <menuitem action="FileSave"/>
      <menuitem action="FileSaveAs"/>
      <menuitem action="FileExportSource"/>
      <separator/>
      <menuitem action="FileQuit"/>
    </menu>
    <menu action="MenuEdit">
      <menuitem action="EditUndo"/>
      <menuitem action="EditRedo"/>
    </menu>
    <menu action="MenuInsert">
      <menuitem action="InsertLayer"/>
      <menuitem action="InsertSpecies"/>
      <menuitem action="InsertParameter"/>
      <menuitem action="InsertProcess"/>
    </menu>
    <menu action="MenuHelp">
      <menuitem action="HelpAbout"/>
    </menu>
  </menubar>
</ui>
"""


def verbose(func):
    """Debugging helper that allows to track input and output of function
    via decoration"""
    print("monitor %r" % (func.__name__), file=sys.stderr)

    def wrapper_func(*args, **kwargs):
        """The wrapping function
        """
        print("call(\033[0;31m%s.%s\033[0;30m): %r\n" % \
                (type(args[0]).__name__, func.__name__, args[1:]), \
                sys.stderr.flush(), file=sys.stderr)
        ret = func(*args, **kwargs)
        print("    ret(%s): \033[0;32m%r\033[0;30m\n" % \
                 (func.__name__, ret), file=sys.stderr)
        return ret
    return wrapper_func


class GTKProject(SlaveDelegate):
    """A facade of kmcos.types.Project so that
    pygtk can display in a TreeView.
    """

    def __init__(self, parent, menubar):
        self.project_data = ObjectTree([Column('name',
                                               use_markup=True,
                                               data_type=str,
                                               sorted=True),
                                        Column('info')])

        self.project_data.connect('row-activated', self.on_row_activated)
        self.model_tree = Project()
        self._set_treeview_hooks()

        self.menubar = menubar

        self.set_parent(parent)

        self.filename = ''

        self.undo_stack = UndoStack(
            self.model_tree.__repr__,
            self.import_file,
            self.project_data.select,
            menubar,
            self.meta,
            'Initialization')

        SlaveDelegate.__init__(self, toplevel=self.project_data)

    def _set_treeview_hooks(self):
        """Fudge function to import to access function to kmcos.types.Project
        to kmcos.gui.GTKProject.
        """
        self.project_data.clear()
        # Meta
        self.meta = self.project_data.append(None, self.model_tree.meta)
        self.model_tree.meta = self.meta

        # Layer List
        self.model_tree.add_layer = self.add_layer
        self.layer_list = self.project_data.append(None,
                                   self.model_tree.layer_list)
        self.get_layers = lambda: \
            sorted(self.project_data.get_descendants(self.layer_list),
                   key=lambda x: x.name)
        self.model_tree.get_layers = self.get_layers
        self.lattice = self.layer_list

        # Parameter List
        self.parameter_list = self.project_data.append(None,
                                       self.model_tree.parameter_list)
        self.add_parameter = lambda parameter: \
            self.project_data.append(self.parameter_list, parameter)
        self.model_tree.add_parameter = self.add_parameter
        self.get_parameters = lambda: \
            sorted(self.project_data.get_descendants(self.parameter_list),
                   key=lambda x: x.name)
        self.model_tree.get_parameters = self.get_parameters

        # Species List
        self.species_list = self.project_data.append(None,
                                   self.model_tree.species_list)
        self.add_species = lambda species: \
            self.project_data.append(self.species_list, species)
        self.model_tree.add_species = self.add_species
        self.get_speciess = lambda: \
            sorted(self.project_data.get_descendants(self.species_list),
                   key=lambda x: x.name)
        self.model_tree.get_speciess = self.get_speciess

        # Process List
        self.process_list = self.project_data.append(None,
                                     self.model_tree.process_list)
        self.add_process = lambda process:\
            self.project_data.append(self.process_list, process)
        self.model_tree.add_process = self.add_process
        self.get_processes = lambda: \
            sorted(self.project_data.get_descendants(self.process_list),
                   key=lambda x: x.name)
        self.model_tree.get_processes = self.get_processes

        # Output List
        self.output_list = self.project_data.append(None,
                                self.model_tree.output_list)
        self.add_output = lambda output:\
            self.project_data.append(self.output_list, output)
        self.model_tree.add_output = self.add_output
        self.get_outputs = lambda:  \
            sorted(self.project_data.get_descendants(self.output_list),
                   key=lambda x: x.name)
        self.model_tree.get_outputs = self.get_outputs

    def add_layer(self, layer):
        self.project_data.append(self.layer_list, layer)
        if len(self.get_layers()) == 1:
            self.set_default_layer(layer.name)
            self.set_substrate_layer(layer.name)
        return layer

    def set_default_species(self, species):
        self.model_tree.species_list.default_species = species

    def set_substrate_layer(self, layer):
        self.model_tree.layer_list.substrate_layer = layer

    def set_default_layer(self, layer):
        self.model_tree.layer_list.default_layer = layer

    def update(self, model):
        """Update the object tree."""
        self.project_data.update(model)

    def on_row_activated(self, _tree, data):
        if isinstance(data, Layer):
            data.active = not data.active

    def get_name(self):
        """Return project name."""
        if self.filename:
            return os.path.basename(self.filename)
        else:
            return 'Untitled'

    def __repr__(self):
        return str(self.model_tree)

    def import_file(self, filename):
        """Import XML project file into editor GUI,
        unfolding the object tree.

        """
        self.filename = filename
        self.model_tree.import_file(filename)
        self.expand_all()

    def expand_all(self):
        """Expand all list of the project tree
        """
        self.project_data.expand(self.species_list)
        self.project_data.expand(self.layer_list)
        self.project_data.expand(self.parameter_list)
        self.project_data.expand(self.process_list)
        self.project_data.expand(self.output_list)

    def on_key_press(self, _, event):
        """When the user hits the keyboard focusing the treeview
        this event is triggered. Right now the only supported function
        is to deleted the selected item
        """
        selection = self.project_data.get_selected()
        if gtk.gdk.keyval_name(event.keyval) == 'Delete':
            if(isinstance(selection, Species)
            or isinstance(selection, Process)
            or isinstance(selection, Parameter)
            or isinstance(selection, Layer)):
                if kiwi.ui.dialogs.yesno(
                    "Do you really want to delete '%s'?" \
                        % selection.name) == gtk.RESPONSE_YES:
                    self.project_data.remove(selection)

    def on_project_data__selection_changed(self, _, elem):
        """When a new item is selected in the treeview this function
        loads the main area of the window with the corresponding form
        and data.
        """
        slave = self.get_parent().get_slave('workarea')
        if slave:
            self.get_parent().detach_slave('workarea')
        if isinstance(elem, Layer):
            if self.meta.model_dimension in [1, 3]:
                self.get_parent().toast('Only 2d supported')
                return
            self.undo_stack.start_new_action('Edit Layer %s' % elem.name,
                                                               elem)
            form = LayerEditor(elem, self)
            self.get_parent().attach_slave('workarea', form)
            form.focus_topmost()
        elif isinstance(elem, Meta):
            self.undo_stack.start_new_action('Edit Meta', elem)
            meta_form = MetaForm(self.meta, self)
            self.get_parent().attach_slave('workarea', meta_form)
            meta_form.focus_toplevel()
            meta_form.focus_topmost()
        elif isinstance(elem, OutputList):
            self.undo_stack.start_new_action('Edit Output', elem)
            form = OutputForm(self.output_list, self)
            self.get_parent().attach_slave('workarea', form)
            form.focus_topmost()
        elif isinstance(elem, Parameter):
            self.undo_stack.start_new_action('Edit Parameter %s' % elem.name,
                                                                   elem)
            form = ParameterForm(elem, self)
            self.get_parent().attach_slave('workarea', form)
            form.focus_topmost()
        elif isinstance(elem, Process):
            if self.meta.model_dimension in [1, 3]:
                self.get_parent().toast('Only 2d supported')
                return
            self.undo_stack.start_new_action('Edit Process %s' % elem.name,
                                                                 elem)
            form = ProcessForm(elem, self)
            self.get_parent().attach_slave('workarea', form)
            form.focus_topmost()
        elif isinstance(elem, ProcessList):
            if self.meta.model_dimension in [1, 3]:
                self.get_parent().toast('Only 2d supported')
                return
            self.undo_stack.start_new_action('Batch process editing', elem)
            form = BatchProcessForm(self)
            self.get_parent().attach_slave('workarea', form)
            form.focus_topmost()
        elif isinstance(elem, Species):
            self.undo_stack.start_new_action('Edit species', elem)
            form = SpeciesForm(elem, self.project_data)
            self.get_parent().attach_slave('workarea', form)
            form.focus_topmost()
        elif isinstance(elem, SpeciesList):
            self.undo_stack.start_new_action('Edit default species', elem)
            form = SpeciesListForm(elem, self)
            self.get_parent().attach_slave('workarea', form)
            form.focus_topmost()
        elif isinstance(elem, LayerList):
            self.undo_stack.start_new_action('Edit lattice', elem)
            dimension = self.meta.model_dimension
            form = LatticeForm(elem, dimension, self)
            self.get_parent().attach_slave('workarea', form)
            form.focus_topmost()
        else:
            self.get_parent().toast('Not implemented, yet(%s).' % type(elem))


class UndoStack():
    """Work in progress attempt to have a 'back' button
    for the editor.

    """

    def __init__(self, get_state_cb, set_state_from_file_cb,
                    select_elem_cb, menubar, elem, action=''):
        self.menubar = menubar
        self.get_state_cb = get_state_cb
        self.set_state_from_file_cb = set_state_from_file_cb
        self.select_elem_cb = select_elem_cb
        actions = gtk.ActionGroup('Actions')
        actions.add_actions([
        ('EditUndo', None, '_Undo', '<control>Z', 'Undo the last edit',
                                                            self.undo),
        ('EditRedo', None, '_Redo', '<control>Y', 'Redo and undo',
                                                           self.redo)])
        menubar.insert_action_group(actions, 0)
        self.menubar.ensure_update()
        self.stack = []
        self.head = -1
        self.current_action = action
        self.current_elem = elem
        self.get_state_cb = get_state_cb
        self.origin = self.get_state_cb()
        self.state = self.get_state_cb()
        self.menubar.get_widget(
                        '/MainMenuBar/MenuEdit/EditUndo').set_sensitive(False)
        self.menubar.get_widget(
                        '/MainMenuBar/MenuEdit/EditRedo').set_sensitive(False)

    def _set_state_cb(self, string):
        tmpfile = io.StringIO()
        tmpfile.write(string)
        tmpfile.seek(0)
        self.set_state_from_file_cb(tmpfile)

    def start_new_action(self, action, elem):
        """Puts a new diff on the stack of actions."""
        #if self.get_state_cb() != self.state:
            #self.head += 1
            #self.stack = self.stack[:self.head] + [{
                #'action':self.current_action,
                #'state':self.get_state_cb(),
                #'elem':self.current_elem,
                #}]
            #self.state = self.get_state_cb()
        self.current_action = action
        self.current_elem = elem
        self.menubar.get_widget(
            '/MainMenuBar/MenuEdit/EditUndo').set_label('Undo %s' % action)
        self.menubar.get_widget(
            '/MainMenuBar/MenuEdit/EditUndo').set_sensitive(True)
        self.menubar.get_widget(
            '/MainMenuBar/MenuEdit/EditRedo').set_label('Redo')
        self.menubar.get_widget(
            '/MainMenuBar/MenuEdit/EditRedo').set_sensitive(False)

    def undo(self, _):
        """Undo one action."""
        if self.head < 0:
            return
        if self.state != self.get_state_cb():
            # if unstashed changes, first undo those
            self.start_new_action(self.current_action, self.get_state_cb())
            self.head += -1

        self.head += -1
        self.state = self.stack[self.head]['state']
        self._set_state_cb(self.state)

        self.current_action = self.stack[self.head + 1]['action']
        self.current_elem = self.stack[self.head + 1]['elem']

        self.menubar.get_widget(
            '/MainMenuBar/MenuEdit/EditUndo').set_label(
                'Undo %s' % self.stack[self.head]['action'])
        if self.head <= 0:
            self.menubar.get_widget(
                '/MainMenuBar/MenuEdit/EditUndo').set_sensitive(False)
        self.menubar.get_widget(
            '/MainMenuBar/MenuEdit/EditRedo').set_label(
                'Redo %s' % (self.stack[self.head + 1]['action']))
        self.menubar.get_widget(
            '/MainMenuBar/MenuEdit/EditRedo').set_sensitive(True)

    def redo(self, _):
        """Repeat an undone action."""
        if self.head >= len(self.stack) - 1:
            return UserWarning('TopReached')
        else:
            self.head += 1
            self.state = self.stack[self.head]['state']
            self._set_state_cb(self.state)
            self.current_action = self.stack[self.head]['action']
            self.current_elem = self.stack[self.head]['elem']
            #self.select_elem_cb(self.current_elem)

        self.menubar.get_widget(
            '/MainMenuBar/MenuEdit/EditUndo').set_label(
                self.stack[self.head]['action'])
        self.menubar.get_widget(
            '/MainMenuBar/MenuEdit/EditUndo').set_sensitive(True)
        self.menubar.get_widget(
            '/MainMenuBar/MenuEdit/EditRedo').set_label('Redo')
        self.menubar.get_widget(
            '/MainMenuBar/MenuEdit/EditRedo').set_sensitive(False)


class Editor(GladeDelegate):
    """The editor GUI frontend."""
    widgets = ['workarea', 'statbar', 'vbox1']
    gladefile = GLADEFILE
    toplevel_name = 'main_window'

    def __init__(self):
        GladeDelegate.__init__(self, delete_handler=self.on_btn_quit__clicked)

        # Prepare and fill the menu from XML layout
        self.menubar = gtk.UIManager()
        if gtk.pygtk_version < (2, 12):
            self.set_tip = gtk.Tooltips().set_tip
        actions = gtk.ActionGroup('Actions')
        actions.add_actions([
        ('MenuFile', None, '_File'),
        ('FileNew', None, '_New', '<control>N', 'Start new project',
            self.on_btn_new_project__clicked),
        ('FileOpenProject', None, '_Open', '<control>O', 'Open project',
            self.on_btn_open_model__clicked),
        ('FileSave', None, '_Save', '<control>S', 'Save model',
            self.on_btn_save_model__clicked),
        ('FileSaveAs', None, 'Save _As', '<control><shift>s', 'Save model As',
            self.on_btn_save_as__clicked),
        ('FileExportSource', None, '_Export Source',
            '<control>E', 'Export model to Fortran 90 source code',
            self.on_btn_export_src__clicked),
        ('FileQuit', None, '_Quit', '<control>Q', 'Quit the program',
            self.on_btn_quit__clicked),
        ('MenuEdit', None, '_Edit'),
        ('MenuInsert', None, '_Insert'),
        ('InsertParameter', None, 'Para_meter',
            '<control><shift>M', 'Add a new parameter',
            self.on_btn_add_parameter__clicked),
        ('InsertLayer', None, '_Layer',
            '<control><shift>L', 'Add a new layer',
            self.on_btn_add_layer__clicked),
        ('InsertProcess', None, '_Process', '<control><shift>P',
            'Add a new process', self.on_btn_add_process__clicked),
        ('InsertSpecies', None, '_Species', '<control><shift>E',
            'Add a new species', self.on_btn_add_species__clicked),
        ('MenuHelp', None, '_Help'),
        ('HelpAbout', None, '_About'),
        ])

        self.menubar.insert_action_group(actions, 0)
        try:
            mergeid = self.menubar.add_ui_from_string(menu_layout)
        except gobject.GError as error:
            print('Building menu failed: %s, %s' % (error, mergeid))

        # Initialize the project tree, passing in the menu bar
        self.project_tree = GTKProject(parent=self, menubar=self.menubar)
        self.main_window.add_accel_group(self.menubar.get_accel_group())
        self.attach_slave('overviewtree', self.project_tree)
        self.set_title('%s - kmcos' % self.project_tree.get_name())
        self.project_tree.show()

        wid = self.project_tree.menubar.get_widget('/MainMenuBar')
        self.menu_box.pack_start(wid, False, False, 0)
        self.menu_box.show()
        #self.quickbuttons.hide()

        self.saved_state = str(self.project_tree)
        # Cast initial message
        self.toast('Welcome!')

    def add_defaults(self):
        """This function adds some useful defaults that are probably
        needed in every simulation.
        """
        # add dimension
        self.project_tree.meta.add({'model_dimension': '2'})

        # add layer
        default_layer_name = 'default'
        default_layer = Layer(name=default_layer_name,)
        self.project_tree.add_layer(default_layer)
        self.project_tree.lattice.default_layer = default_layer_name

        # add an empty species
        empty_species = 'empty'
        empty = Species(name=empty_species, color='#fff')
        # set empty as default species
        self.project_tree.species_list.default_species = empty_species
        self.project_tree.add_species(empty)
        # add standard parameter
        param = Parameter(name='lattice_size', value='40 40 1')
        self.project_tree.add_parameter(param)

        param = Parameter(name='print_every', value='1.e5')
        self.project_tree.add_parameter(param)

        param = Parameter(name='total_steps', value='1.e7')
        self.project_tree.add_parameter(param)

        # add output entries
        self.project_tree.add_output(OutputItem(name='kmc_time',
                                                output=True))
        self.project_tree.add_output(OutputItem(name='walltime',
                                                output=False))
        self.project_tree.add_output(OutputItem(name='kmc_step',
                                                output=False))

        self.project_tree.expand_all()

    def toast(self, toast):
        """Present a nice little text in the middle of the workarea
        as a standard way write inline messages to the user
        """
        if self.get_slave('workarea'):
            self.detach_slave('workarea')
        inline_message = InlineMessage(toast)
        self.attach_slave('workarea', inline_message)
        inline_message.show()

    def on_btn_new_project__clicked(self, _button):
        """Start a new project
        """
        if str(self.project_tree) != self.saved_state:
            # if there are unsaved changes, ask what to do first
            save_changes_dialog = gtk.Dialog(
                            buttons=(gtk.STOCK_DISCARD,
                                     gtk.RESPONSE_DELETE_EVENT,
                                     gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_SAVE, gtk.RESPONSE_OK),
                                     title='Saved unsaved changes?')
            save_changes_dialog.vbox.pack_start(
                gtk.Label(
                    "\nThere are unsaved changes.\nWhat shall we do?\n\n"))
            save_changes_dialog.show_all()
            resp = save_changes_dialog.run()
            save_changes_dialog.destroy()
            if resp == gtk.RESPONSE_DELETE_EVENT:
                # nothing to do here
                pass
            elif resp == gtk.RESPONSE_CANCEL:
                return
            elif resp == gtk.RESPONSE_OK:
                self.on_btn_save_model__clicked(None)
        # Instantiate new project data
        self.project_tree = GTKProject(parent=self)
        if self.get_slave('overviewtree'):
            self.detach_slave('overviewtree')
        self.attach_slave('overviewtree', self.project_tree)
        self.project_tree.show()
        self.toast(
            'Start a new project by filling in meta information,\n' +
            'lattice, species, parameters, and processes or open\n' +
            'an existing one by opening a kMC XML file')

    def on_btn_add_layer__clicked(self, _button):
        """Add a new layer to the model
        """
        if len(self.project_tree.layer_list) == 1:
            kiwi.ui.dialogs.warning('Entering multi-lattice mode',
                int='This is an unpublished feature\n' +
                'Please ask me about publishing results obtained\n' +
                'from using this feature mjhoffmann@gmail.com')
        if self.project_tree.meta.model_dimension in [1, 3]:
            self.toast('Only 2d supported')
            return
        new_layer = Layer()
        self.project_tree.undo_stack.start_new_action('Add layer', new_layer)
        self.project_tree.add_layer(new_layer)
        layer_form = LayerEditor(new_layer, self.project_tree)
        self.project_tree.project_data.expand(self.project_tree.layer_list)
        if self.get_slave('workarea'):
            self.detach_slave('workarea')
        self.attach_slave('workarea', layer_form)
        layer_form.focus_topmost()

    def on_btn_add_species__clicked(self, _button):
        """Add a new species to the model
        """
        new_species = Species(color='#fff', name='')
        self.project_tree.undo_stack.start_new_action('Add species',
                                                      new_species)
        self.project_tree.add_species(new_species)
        self.project_tree.project_data.expand(self.project_tree.species_list)
        self.project_tree.project_data.select(new_species)
        species_form = SpeciesForm(new_species, self.project_tree)
        if self.get_slave('workarea'):
            self.detach_slave('workarea')
        self.attach_slave('workarea', species_form)
        species_form.focus_topmost()

    def on_btn_add_process__clicked(self, _button):
        """Add a new process to the model
        """
        if self.project_tree.meta.model_dimension in [1, 3]:
            self.toast('Only 2d supported')
            return
        if not self.project_tree.get_layers():
            self.toast("No layer defined, yet!")
            return
        new_process = Process(name='', rate_constant='')
        self.project_tree.undo_stack.start_new_action('Add process',
                                                      new_process)
        self.project_tree.add_process(new_process)
        self.project_tree.project_data.expand(self.project_tree.process_list)
        self.project_tree.project_data.select(new_process)
        process_form = ProcessForm(new_process, self.project_tree)
        if self.get_slave('workarea'):
            self.detach_slave('workarea')
        self.attach_slave('workarea', process_form)
        process_form.focus_topmost()

    def on_btn_add_parameter__clicked(self, _button):
        new_parameter = Parameter(name='', value='')
        self.project_tree.undo_stack.start_new_action('Add parameter',
                                                      new_parameter)
        self.project_tree.add_parameter(new_parameter)
        self.project_tree.project_data.expand(
                                       self.project_tree.parameter_list)
        self.project_tree.project_data.select(new_parameter)
        parameter_form = ParameterForm(new_parameter, self.project_tree)
        if self.get_slave('workarea'):
            self.detach_slave('workarea')
        self.attach_slave('workarea', parameter_form)
        parameter_form.focus_topmost()

    def on_btn_open_model__clicked(self, _button):
        """Import project from XML
        """
        if str(self.project_tree) != self.saved_state:
            # if there are unsaved changes, ask what to do first
            save_changes_dialog = gtk.Dialog(
                        buttons=(gtk.STOCK_DISCARD,
                                 gtk.RESPONSE_DELETE_EVENT,
                                 gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                 gtk.STOCK_SAVE, gtk.RESPONSE_OK),
                                 title='Saved unsaved changes?')
            save_changes_dialog.vbox.pack_start(
                gtk.Label(
                    "\nThere are unsaved changes.\nWhat shall we do?\n\n"))
            save_changes_dialog.show_all()
            resp = save_changes_dialog.run()
            save_changes_dialog.destroy()
            if resp == gtk.RESPONSE_DELETE_EVENT:
                # nothing to do here
                pass
            elif resp == gtk.RESPONSE_CANCEL:
                return
            elif resp == gtk.RESPONSE_OK:
                self.on_btn_save_model__clicked(None)

        # choose which file to open next
        filechooser = gtk.FileChooserDialog(title='Open Project',
            action=gtk.FILE_CHOOSER_ACTION_OPEN,
            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                     gtk.STOCK_OK, gtk.RESPONSE_OK))
        resp = filechooser.run()
        filename = filechooser.get_filename()
        filechooser.destroy()
        if resp == gtk.RESPONSE_OK and filename:
            # Initialize blank project tree
            self.project_tree = GTKProject(parent=self, menubar=self.menubar)
            if self.get_slave('overviewtree'):
                self.detach_slave('overviewtree')
            self.attach_slave('overviewtree', self.project_tree)
            self.set_title('%s - kmcos' % self.project_tree.get_name())
            self.project_tree.show()
            self.import_file(filename)

    def import_xml_file(self, filename):
        """Stub for legacy support."""
        return self.import_file(filename)

    def import_file(self, filename):
        self.project_tree._set_treeview_hooks()
        # Import
        self.project_tree.import_file(filename)
        self.set_title('%s - kmcos' % self.project_tree.get_name())
        if hasattr(self.project_tree.meta, 'model_name'):
            self.toast('Imported model %s' %
                       self.project_tree.meta.model_name)
        else:
            self.toast('Imported model <Untitled>')
        self.saved_state = str(self.project_tree)

    def on_btn_save_model__clicked(self, _button, force_save=False):
        #Write Out XML File
        xml_string = str(self.project_tree)
        if xml_string == self.saved_state and not force_save:
            self.toast('Nothing to save')
        else:
            if not self.project_tree.filename:
                self.on_btn_save_as__clicked(None)
            #outfile = open(self.project_tree.filename, 'w')
            #outfile.write(xml_string)
            #outfile.write('<!-- This is an automatically generated XML ' +
                          #'file, representing a kMC model ' +
                          #'please do not change this unless ' +
                          #'you know what you are doing -->\n')
            #outfile.close()
            self.project_tree.model_tree.save(self.project_tree.filename)
            self.saved_state = xml_string
            self.toast('Saved %s' % self.project_tree.filename)

    def on_btn_save_as__clicked(self, _button):
        filechooser = gtk.FileChooserDialog(title='Save Project As ...',
            action=gtk.FILE_CHOOSER_ACTION_SAVE,
            parent=None,
            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                     gtk.STOCK_OK, gtk.RESPONSE_OK))
        filechooser.set_property('do-overwrite-confirmation', True)
        resp = filechooser.run()
        if resp == gtk.RESPONSE_OK:
            self.project_tree.filename = filechooser.get_filename()
            self.on_btn_save_model__clicked(None, force_save=True)
        filechooser.destroy()

    #@verbose
    def on_btn_export_src__clicked(self, _button, export_dir=''):
        self.toast('Exporting source code ...')
        if not export_dir:
            export_dir = kiwi.ui.dialogs.selectfolder(
                title='Select folder for F90 source code.')
        if not export_dir:
            self.toast('No folder selected.')
            return

        kmcos.io.export_source(self.project_tree, export_dir)

        # return directory name
        self.toast('Wrote FORTRAN sources to %s\n' % export_dir +
            'Please go to the directory and run "kmcos build".\n' +
           'If this finished successfully you can run the simulation\n' +
           'by executing "kmcos view"')

    def on_btn_help__clicked(self, _button):
        """Preliminary help function."""
        help_url = 'http://mhoffman.github.com/kmcos/doc/build/html/index.html'
        issues_url = 'https://github.com/mhoffman/kmcos/issues'
        gtk.show_uri(None, help_url, gtk.gdk.CURRENT_TIME)
        self.toast(('Please refer to online help at\n%s.\n\n'
                    'Or post issues at\n%s.') %
                    (help_url, issues_url))

    def on_btn_quit__clicked(self, _button, *_args):
        """Checks if unsaved changes. If so offer file save dialog.
           Otherwise quit directly.

        """
        if self.saved_state != str(self.project_tree):
            save_changes_dialog = gtk.Dialog(
                      buttons=(gtk.STOCK_DISCARD, gtk.RESPONSE_DELETE_EVENT,
                      gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                      gtk.STOCK_SAVE, gtk.RESPONSE_OK),
                      title='Saved unsaved changes?')
            save_changes_dialog.vbox.pack_start(
                gtk.Label(
                    "\nThere are unsaved changes.\nWhat shall we do?\n\n"))
            save_changes_dialog.show_all()
            resp = save_changes_dialog.run()
            save_changes_dialog.destroy()
            if resp == gtk.RESPONSE_CANCEL:
                return True
            elif resp == gtk.RESPONSE_DELETE_EVENT:
                self.hide_and_quit()
                gtk.main_quit()
            elif resp == gtk.RESPONSE_OK:
                self.on_btn_save_model__clicked(None)
                self.hide_and_quit()
                gtk.main_quit()
        else:
            self.hide_and_quit()
            gtk.main_quit()
        # Need to return true, or otherwise the window manager destroys
        # the windows anyways
        return True


def main():
    """Main entry point to GUI Editor."""
    parser = optparse.OptionParser()
    parser.add_option('-o', '--open',
                      dest='xml_file',
                      help='Immediately import kmcos XML file')
    parser.add_option('-x', '--export-dir',
                      dest='export_dir',
                      type=str)
    (options, args) = parser.parse_args()
    editor = kmcos.gui.Editor()
    if len(args) >= 2:
        options.xml_file = args[1]

    if options.xml_file:
        editor.import_file(options.xml_file)
        editor.toast('Imported %s' % options.xml_file)
    else:
        print('No XML file provided, starting a new model.')
        editor.add_defaults()
        editor.saved_state = str(editor.project_tree)
    if hasattr(options, 'export_dir') and options.export_dir:
        print('Exporting right-away')
        editor.on_btn_export_src__clicked(_button='',
                                          export_dir=options.export_dir)
        exit()
    editor.show_and_loop()



import numpy as np

"""
A  module to run the acf features for a compiled kMC model.

The model can be used directly like so::

    import kmcos.run.acf as acf
    from kmcos.run import KMC_Model
    import kmcos.run.acf as acf

    model = KMC_Model()

    nr_of_steps = 1e7

    nr_of_types = 2

    t_bin = 0.0005
    t_f = 0.022
    safety_factor = 1
    extending_factor = 3 

    types = [0.5,1]
    site_types = [['default_a_1','default_a_2','default_b_1','default_b_2'],[1,1,2,2]]

    acf.allocate_acf(model,nr_of_types,t_bin,t_f,safety_factor,extending_factor)

    for i in range(len(types)):
  	 acf.set_types_acf(model,types[i])

    acf.calc_product_property(model)

    for i in range(len(site_types[0])):
    	acf.set_property_acf(model,site_types[0][i],site_types[1][i])

    acf.initialize_acf(model,species = 'ion')

    acf.do_kmc_steps_acf(model,nr_of_steps)

    ACF = acf.get_acf(model,normalization = False)

which, of course can also be part of a python script.
"""

#    Copyright 2015-2016 Andreas Garhammer
#    This file is part of kmcos.

def get_id_arr(kmc_model):
    """Return the id's from id_arr.
    """
    id_arr = np.zeros((kmc_model.base.get_volume()))
    for i in range(kmc_model.base.get_volume()):
        id_arr[i] = kmc_model.base_acf.get_id_arr(i + 1)
    return id_arr


def get_site_arr(kmc_model):
    """Return the site indices from site_arr.
    """
    site_arr = np.zeros((kmc_model.base.get_volume()))
    for i in range(kmc_model.base.get_volume()):
        site_arr[i] = kmc_model.base_acf.get_site_arr(i + 1)
    return site_arr


def get_property_o(kmc_model):
    """Return the property_o (g(0)) for each tracked particle
    from property_o.
    """
    property_o = np.zeros((kmc_model.base.get_volume()))
    for i in range(kmc_model.base.get_volume()):
        property_o[i] = kmc_model.base_acf.get_property_o(i + 1)
    return property_o


def get_property_acf(kmc_model):
    """Return the type indices for each site from property_acf.
    """
    property_acf = np.zeros((kmc_model.base.get_volume()))
    for i in range(kmc_model.base.get_volume()):
        property_acf[i] = kmc_model.base_acf.get_property_acf(i + 1)
    return property_acf


def get_buffer_acf(kmc_model):
    """Return the product of properties (g(0)(g(t))) for each
    tracked particle from buffer_acf.
    """
    buffer_acf = np.zeros((kmc_model.base.get_volume()))
    for i in range(kmc_model.base.get_volume()):
        buffer_acf[i] = kmc_model.base_acf.get_buffer_acf(i + 1)
    return buffer_acf


def get_config_bin_acf(kmc_model):
    """Return the entries for each bin from config_bin.
    """
    config_bin = np.zeros((kmc_model.base_acf.extended_nr_of_bins))
    for i in range(kmc_model.base_acf.extended_nr_of_bins):
        config_bin[i] = kmc_model.base_acf.get_config_bin_acf(i + 1)
    return config_bin


def get_counter_write_in_bin_acf(kmc_model):
    """Return the number of contributions to each bin from
    counter_write_in_bin.
    """
    contribution_bin = np.zeros((kmc_model.base_acf.extended_nr_of_bins))
    for i in range(kmc_model.base_acf.extended_nr_of_bins):
        contribution_bin[
            i] = kmc_model.base_acf.get_counter_write_in_bin(i + 1)
    return contribution_bin


def get_acf(kmc_model):
    """Returns the ACF.
    """
    acf = np.zeros((kmc_model.base_acf.nr_of_bins))
    for i in range(kmc_model.base_acf.nr_of_bins):
        acf[i] = kmc_model.base_acf.calc_acf(i + 1)
    return acf


def get_types_acf(kmc_model):
    """Return the properties of each type from
    from types.
    """
    types_arr = np.zeros((kmc_model.base_acf.nr_of_types))
    for i in range(kmc_model.base_acf.nr_of_types):
        types_arr[i] = kmc_model.base_acf.get_types(i + 1)
    return types_arr


def get_product_property(kmc_model):
    """Return the matrix with all possible combinations of products
    between two properties (g(0)g(t)).
    """
    product_property = np.zeros(
        (1, kmc_model.base_acf.nr_of_types, kmc_model.base_acf.nr_of_types))
    for i in range(kmc_model.base_acf.nr_of_types):
        for j in range(kmc_model.base_acf.nr_of_types):
            product_property[
                0, i, j] = kmc_model.base_acf.get_product_property(i + 1, j + 1)
    return product_property


def get_trajectory(kmc_model):
    """Return the trajectory for each tracked particle from
    from trajectory.
    """
    trajectory = np.zeros(
        (1, kmc_model.base_acf.nr_of_ions, kmc_model.base_acf.nr_of_steps + 1),
        'int')
    for i in range(kmc_model.base_acf.nr_of_ions):
        for j in range(kmc_model.base_acf.nr_of_steps + 1):
            trajectory[0, i, j] = kmc_model.base_acf.get_trajectory(
                i + 1, j + 1)
    return trajectory


def get_displacement(kmc_model):
    """Return the displacement for each tracked particle from
    from displacement.
    """
    displacement = np.zeros((1, kmc_model.base_acf.nr_of_ions, 3))
    for i in range(kmc_model.base_acf.nr_of_ions):
        displacement[0, i] = kmc_model.base_acf.get_displacement(i + 1)
    return displacement


def allocate_acf(kmc_model, nr_of_types, t_bin, t_f, safety_factor=None, extending_factor=None):
    """Allocate the whole arrays for the tracking process and for
    the sampling of the ACF.
    """
    kmc_model.base_acf.allocate_tracing_arr(nr_of_types)
    kmc_model.base_acf.allocate_config_bin_acf(
        t_bin, t_f, safety_factor, extending_factor)


def allocate_trajectory(kmc_model, nr_of_steps):
    """Allocates the trajectory array for the recording of the trajectory 
    of each tracked particle. The user has to specify for how many kMC steps
    the trajectory should be recorded. 
    """
    kmc_model.base_acf.allocate_trajectory(nr_of_steps)


def set_types_acf(kmc_model, site_property):
    """Set the properties, which are given by the user to 
       a type index.
    """
    types = get_types_acf(kmc_model)
    for i in range(len(types)):
        if types[i] == 0:
            kmc_model.base_acf.set_types(i + 1, site_property)
            break


def calc_product_property(kmc_model, ):
    """Caculate and set all possible combinations of products
    between two properties(g(0)g(t)).
    """
    types = get_types_acf(kmc_model)
    for i in range(kmc_model.base_acf.nr_of_types):
        for j in range(kmc_model.base_acf.nr_of_types):
            kmc_model.base_acf.set_product_property(
                i + 1, j + 1, types[i] * types[j])


def do_kmc_steps_acf(kmc_model, n, traj_on=False):
    """Performs n kMC steps for the sampling of ACF.
    Recording of the trajectory is optional
    (on: traj_on = True, off: traj_on = False).
    """
    kmc_model.proclist_acf.do_kmc_steps_acf(n, traj_on)


def do_kmc_steps_displacement(kmc_model, n, traj_on=False):
    """Performs n kMC steps for the recording of the displacment
    for each tracked particle.
    Recording of the trajectory is optional
    (on: traj_on = True, off: traj_on = False).
    """
    kmc_model.proclist_acf.do_kmc_steps_displacement(n, traj_on)


def initialize_acf(kmc_model, species):
    """Initialize the whole arrays for the tracking process
    and for the sampling of the ACF for a given species.
    """
    trace_species = getattr(kmc_model.proclist, species.lower())
    kmc_model.base_acf.initialize_acf(trace_species)


def initialize_msd(kmc_model, species):
    """Initialize the whole arrays for the tracking process
    and for the recording of displacement for a given species.
    """
    trace_species = getattr(kmc_model.proclist, species.lower())
    kmc_model.base_acf.initialize_mean_squared_displacement(trace_species)


def calc_msd(kmc_model):
    """Calculates the mean squared displacement (msd) as a particle average
    from the displacements of the tracked particles.
    """
    msd = kmc_model.base_acf.calc_mean_squared_disp()
    return msd


def set_property_acf(kmc_model, layer_site_name, property_type):
    """Set types indices of corresponding properties to sites, for which
    the site_names are given by the user.
    """
    for i in range((kmc_model.base.get_volume())):
        if ((i + 1) % kmc_model.lattice.spuck) + kmc_model.lattice.spuck == getattr(kmc_model.lattice, layer_site_name.lower()):
            kmc_model.base_acf.set_property_acf(i + 1, property_type)
        if (i + 1) % kmc_model.lattice.spuck == getattr(kmc_model.lattice, layer_site_name.lower()):
            kmc_model.base_acf.set_property_acf(i + 1, property_type)


def get_acf(kmc_model, normalization=False):
    """Return the ACF.
    The normalization of the ACF is optional.
    (on: normalization = True, off: normalization = False)
    """
    acf = np.zeros((kmc_model.base_acf.nr_of_bins))
    for i in range(kmc_model.base_acf.nr_of_bins):
        acf[i] = kmc_model.base_acf.calc_acf(i + 1)
    if normalization == True:
        acf = acf / acf[0]
    return acf


def set_acf_to_zero(kmc_model):
    """Set all acf arrays and parameters to initial state
    After this, the sampling of ACF can start from new.
    """
    kmc_model.base_acf.set_acf_to_zero()


def set_displacement_to_zero(kmc_model):
    """Sets the displacement vector to zero.
    After this, the recording of the displacement can start
    from new.
    """
    kmc_model.base_acf.set_displacement_to_zero()



"""
Feature overview
================
With kmcos you can:

    * easily create and modify kMC models through GUI
    * store and exchange kMC models through XML
    * generate fast, platform independent, self-contained code [#code]_
    * run kMC models through GUI or python bindings

kmcos has been developed in the context of first-principles based modelling
of surface chemical reactions but might be of help for other types of
kMC models as well.

kmcos' goal is to significantly reduce the time you need
to implement and run a lattice kmc simulation. However it can not help
you plan the model.


kmcos can be invoked directly from the command line in one of the following
ways::

    kmcos [help] (all|benchmark|build|edit|export|help|import|rebuild|run|settings-export|shell|version|view|xml) [options]

or it may be used as an API via the *kmcos* module.

.. rubric:: Footnotes

.. [#code] The source code is generated in Fortran90, written in a modular
            fashion. Python bindings are generated using `f2py  <http://cens.ioc.ee/projects/f2py2e/>`_.
"""
from __future__ import print_function

#    Copyright 2009-2013 Max J. Hoffmann (mjhoffmann@gmail.com)
#    This file is part of kmcos.
#
#    kmcos is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    kmcos is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with kmcos.  If not, see <http://www.gnu.org/licenses/>.


#import kmcos.types
#import kmcos.io

__version__ = "0.0.20"
VERSION = __version__

def evaluate_param_expression(param, parameters={}):
    import tokenize
    import io
    import math
    from kmcos import units

    # convert parameters to dict if passed as list of Parameters()
    if type(parameters) is list:
        param_dict = {}
        for parameter in parameters:
            param_dict[parameter.name] = {'value': parameter.value}
        parameters = param_dict

    parameter_str = str(parameters[param]['value'])

    # replace some aliases
    parameter_str = parameter_str.replace('beta', '(1./(kboltzmann*T))')
    # replace units used in parameters
    for unit in units.keys:
        parameter_str = parameter_str.replace(
                        unit, '%s' % eval('units.%s' % unit))
    try:
        return eval(parameter_str)
    except:
        try:
            replaced_tokens=[]
            input = io.StringIO(parameter_str).readline
            tokens = list(tokenize.generate_tokens(input))
        except:
            raise Exception('Could not tokenize expression: %s' % input)
        for i, token, _, _, _ in tokens:
            if token in ['sqrt', 'exp', 'sin', 'cos', 'pi', 'pow', 'log']:
                replaced_tokens.append((i, 'math.' + token))
            elif token.startswith('GibbsGas_'):
                #evaluate gas phase gibbs free energy using ase thermochemistry module,
                #experimental data from NIST CCCBDB, the electronic energy 
                #and current temperature and partial pressure
                from kmcos import species
                species_name = '_'.join(token.split('_')[1:])
                if species_name in dir(species):
                    if not 'T' in parameters:
                        raise Exception('Need "T" in parameters to evaluate gas phase gibbs free energy.')

                    if not ('p_%s' % species_name) in parameters:
                        raise Exception('Need "p_%s" in parameters to evaluate gas phase gibbs free energy.' % species_name)

                    replaced_tokens.append((i, 'species.%s.GibbsGas(%s,%s,%s)' % (
                                species_name,
                                parameters['E_'+species_name]['value'],
                                parameters['T']['value'],
                                parameters['p_%s' % species_name]['value'],
                                )))
                else:
                    print('No NIST data assigned for %s' % species_name)
                    print('Setting chemical potential to zero')
                    replaced_tokens.append((i, '0'))

                #gibbs=eval(replaced_tokens2[-1][-1])
                #print species_name+': %.3f'%gibbs

            elif token.startswith('GibbsAds_'):
                #evaluate gibbs free energy of adsorbate using ase thermochemistry module,
                #calculated frequencies and electronic energy and current temperature
                from kmcos import species
                species_name = '_'.join(token.split('_')[1:])
                if not 'T' in parameters:
                    raise Exception('Need "T" in parameters to evaluate adsorbate gibbs free energy.')
                energy=parameters['E_'+species_name]['value']
                try:
                    eval(energy)
                except:
                    try:
                        replaced_tokens2=[]
                        input = io.StringIO(energy).readline
                        tokens2 = list(tokenize.generate_tokens(input))
                    except:
                        raise Exception('Could not tokenize expression: %s' % input)
                    for j, token2, _, _, _ in tokens2:
                        if token2 in parameters:
                            parameter_str2 = str(parameters[token2]['value'])
                            try:
                                eval(parameter_str2)
                                replaced_tokens2.append((j, parameter_str2))
                            except:
                                try:
                                    input = io.StringIO(parameter_str2).readline
                                    tokens3 = list(tokenize.generate_tokens(input))
                                except:
                                    raise Exception('Could not tokenize expression: %s' % input)
                                for k, token3, _, _, _ in tokens3:
                                    if token3 in parameters:
                                        parameter_str3 = str(parameters[token3]['value'])
                                        replaced_tokens2.append((k, parameter_str3))
                                    else:
                                        replaced_tokens2.append((k, token3))
                        else:
                            replaced_tokens2.append((j, token2))
                    energy = tokenize.untokenize(replaced_tokens2)
                replaced_tokens.append((i, 'species.GibbsAds(%s,%s,%s)' % (
                            energy,
                            parameters['f_'+species_name]['value'],
                            parameters['T']['value'],
                            )))

                #gibbs=eval(replaced_tokens2[-1][-1])
                #print species_name+': %.3f'%gibbs

            elif token in parameters:
                replaced_tokens.append((i, str(parameters[token]['value'])))
            else:
                replaced_tokens.append((i, token))
        parameter_str = tokenize.untokenize(replaced_tokens)
        return eval(parameter_str)
        #print parameter_str
        #print token+': %.7f'%eval(parameter_str)

rate_aliases = { 'beta' : '(1/(kboltzmann*T))'}

def evaluate_rate_expression(rate_expr, parameters={}):
    """Evaluates an expression for a typical kMC rate constant.
     External parameters can be passed in as dictionary, like the
     following:
        parameters = {'p_CO':{'value':1},
                      'T':{'value':1}}

     or as a list of parameters:
        parameters = [Parameter(), ... ]
     """
    import tokenize
    import io
    import math
    from kmcos import units

    # convert parameters to dict if passed as list of Parameters()
    if type(parameters) is list:
        param_dict = {}
        for parameter in parameters:
            param_dict[parameter.name] = {'value': parameter.value}
        parameters = param_dict

    if not rate_expr:
        rate_const = 0.0
    else:
        replaced_tokens = []

        # replace some aliases
        for old, new in list(rate_aliases.items()):
            rate_expr = rate_expr.replace(old, new)
        try:
            input = io.StringIO(rate_expr).readline
            tokens = list(tokenize.generate_tokens(input))
        except:
            raise Exception('Could not tokenize expression: %s' % input)
        for i, token, _, _, _ in tokens:
            if token in ['sqrt', 'exp', 'sin', 'cos', 'pi', 'pow', 'log']:
                replaced_tokens.append((i, 'math.' + token))
            elif token in dir(units):
                replaced_tokens.append((i, str(eval('units.' + token))))
            elif token.startswith('m_'):
                from ase.symbols import string2symbols
                from ase.data import atomic_masses
                from ase.data import atomic_numbers
                species_name = '_'.join(token.split('_')[1:])
                symbols = string2symbols(species_name)
                replaced_tokens.append((i,
                            '%s' % sum([atomic_masses[atomic_numbers[symbol]]
                            for symbol in symbols])))
            elif token.startswith('mu_'):
                # evaluate gas phase chemical potential if among
                # available JANAF tables if from current temperature
                # and corresponding partial pressure
                from kmcos import species
                species_name = '_'.join(token.split('_')[1:])
                if species_name in dir(species):
                    if not 'T' in parameters:
                        raise Exception('Need "T" in parameters to evaluate chemical potential.')

                    if not ('p_%s' % species_name) in parameters:
                        raise Exception('Need "p_%s" in parameters to evaluate chemical potential.' % species_name)

                    replaced_tokens.append((i, 'species.%s.mu(%s,%s)' % (
                                   species_name,
                                   parameters['T']['value'],
                                   parameters['p_%s' % species_name]['value'],
                                   )))
                else:
                    print('No JANAF table assigned for %s' % species_name)
                    print('Setting chemical potential to zero')
                    replaced_tokens.append((i, '0'))

            elif token.startswith('GibbsGas_'):
                #evaluate gas phase gibbs free energy using ase thermochemistry module,
                #experimental data from NIST CCCBDB, the electronic energy 
                #and current temperature and partial pressure
                from kmcos import species
                species_name = '_'.join(token.split('_')[1:])
                if species_name in dir(species):
                    if not 'T' in parameters:
                        raise Exception('Need "T" in parameters to evaluate gas phase gibbs free energy.')

                    if not ('p_%s' % species_name) in parameters:
                        raise Exception('Need "p_%s" in parameters to evaluate gas phase gibbs free energy.' % species_name)

                    replaced_tokens.append((i, 'species.%s.GibbsGas(%s,%s,%s)' % (
                                   species_name,
                                   parameters['E_'+species_name]['value'],
                                   parameters['T']['value'],
                                   parameters['p_%s' % species_name]['value'],
                                   )))
                else:
                    print('No NIST data assigned for %s' % species_name)
                    print('Setting chemical potential to zero')
                    replaced_tokens.append((i, '0'))

                #gibbs=eval(replaced_tokens[-1][-1])
                #print species_name+': %.3f'%gibbs

            elif token.startswith('GibbsAds_'):
                #evaluate gibbs free energy of adsorbate using ase thermochemistry module,
                #calculated frequencies and electronic energy and current temperature
                from kmcos import species
                species_name = '_'.join(token.split('_')[1:])
                if not 'T' in parameters:
                    raise Exception('Need "T" in parameters to evaluate adsorbate gibbs free energy.')
                energy=parameters['E_'+species_name]['value']
                try:
                    eval(energy)
                except:
                    try:
                        replaced_tokens2=[]
                        input = io.StringIO(energy).readline
                        tokens2 = list(tokenize.generate_tokens(input))
                    except:
                        raise Exception('Could not tokenize expression: %s' % input)
                    for j, token2, _, _, _ in tokens2:
                        if token2 in parameters:
                            parameter_str = str(parameters[token2]['value'])
                            try:
                                eval(parameter_str)
                                replaced_tokens2.append((j, parameter_str))
                            except:
                                try:
                                    input = io.StringIO(parameter_str).readline
                                    tokens3 = list(tokenize.generate_tokens(input))
                                except:
                                    raise Exception('Could not tokenize expression: %s' % input)
                                for k, token3, _, _, _ in tokens3:
                                    if token3 in parameters:
                                        parameter_str = str(parameters[token3]['value'])
                                        replaced_tokens2.append((k, parameter_str))
                                    else:
                                        replaced_tokens2.append((k, token3))
                        else:
                            replaced_tokens2.append((j, token2))
                    energy = tokenize.untokenize(replaced_tokens2)
                replaced_tokens.append((i, 'species.GibbsAds(%s,%s,%s)' % (
                               energy,
                               parameters['f_'+species_name]['value'],
                               parameters['T']['value'],
                               )))

                #gibbs=eval(replaced_tokens[-1][-1])
                #print species_name+': %.3f'%gibbs

            elif token in parameters:
                parameter_str = str(parameters[token]['value'])
                # replace some aliases
                parameter_str = parameter_str.replace('beta', '(1./(kboltzmann*T))')
                # replace units used in parameters
                for unit in units.keys:
                    parameter_str = parameter_str.replace(
                                    unit, '%s' % eval('units.%s' % unit))
                try:
                    eval(parameter_str)
                    replaced_tokens.append((i, parameter_str))
                except:
                    try:
                        replaced_tokens2=[]
                        input = io.StringIO(parameter_str).readline
                        tokens2 = list(tokenize.generate_tokens(input))
                    except:
                        raise Exception('Could not tokenize expression: %s' % input)
                    for i, token2, _, _, _ in tokens2:
                        if token2 in ['sqrt', 'exp', 'sin', 'cos', 'pi', 'pow', 'log']:
                            replaced_tokens2.append((i, 'math.' + token2))
                        elif token2.startswith('GibbsGas_'):
                            #evaluate gas phase gibbs free energy using ase thermochemistry module,
                            #experimental data from NIST CCCBDB, the electronic energy 
                            #and current temperature and partial pressure
                            from kmcos import species
                            species_name = '_'.join(token2.split('_')[1:])
                            if species_name in dir(species):
                                if not 'T' in parameters:
                                    raise Exception('Need "T" in parameters to evaluate gas phase gibbs free energy.')

                                if not ('p_%s' % species_name) in parameters:
                                    raise Exception('Need "p_%s" in parameters to evaluate gas phase gibbs free energy.' % species_name)

                                replaced_tokens2.append((i, 'species.%s.GibbsGas(%s,%s,%s)' % (
                                            species_name,
                                            parameters['E_'+species_name]['value'],
                                            parameters['T']['value'],
                                            parameters['p_%s' % species_name]['value'],
                                            )))
                            else:
                                print('No NIST data assigned for %s' % species_name)
                                print('Setting chemical potential to zero')
                                replaced_tokens2.append((i, '0'))

                            #gibbs=eval(replaced_tokens2[-1][-1])
                            #print species_name+': %.3f'%gibbs

                        elif token2.startswith('GibbsAds_'):
                            #evaluate gibbs free energy of adsorbate using ase thermochemistry module,
                            #calculated frequencies and electronic energy and current temperature
                            from kmcos import species
                            species_name = '_'.join(token2.split('_')[1:])
                            if not 'T' in parameters:
                                raise Exception('Need "T" in parameters to evaluate adsorbate gibbs free energy.')
                            energy=parameters['E_'+species_name]['value']
                            try:
                                eval(energy)
                            except:
                                try:
                                    replaced_tokens3=[]
                                    input = io.StringIO(energy).readline
                                    tokens3 = list(tokenize.generate_tokens(input))
                                except:
                                    raise Exception('Could not tokenize expression: %s' % input)
                                for j, token3, _, _, _ in tokens3:
                                    if token3 in parameters:
                                        parameter_str = str(parameters[token3]['value'])
                                        try:
                                            eval(parameter_str)
                                            replaced_tokens3.append((j, parameter_str))
                                        except:
                                            try:
                                                input = io.StringIO(parameter_str).readline
                                                tokens4 = list(tokenize.generate_tokens(input))
                                            except:
                                                raise Exception('Could not tokenize expression: %s' % input)
                                            for k, token4, _, _, _ in tokens4:
                                                if token4 in parameters:
                                                    parameter_str = str(parameters[token4]['value'])
                                                    replaced_tokens3.append((k, parameter_str))
                                                else:
                                                    replaced_tokens3.append((k, token4))
                                    else:
                                        replaced_tokens3.append((j, token3))
                                energy = tokenize.untokenize(replaced_tokens3)
                            replaced_tokens2.append((i, 'species.GibbsAds(%s,%s,%s)' % (
                                        energy,
                                        parameters['f_'+species_name]['value'],
                                        parameters['T']['value'],
                                        )))

                            #gibbs=eval(replaced_tokens2[-1][-1])
                            #print species_name+': %.3f'%gibbs

                        elif token2 in parameters:
                            replaced_tokens2.append((i, str(parameters[token2]['value'])))
                        else:
                            replaced_tokens2.append((i, token2))
                    parameter_str = tokenize.untokenize(replaced_tokens2)

                    #print parameter_str
                    #print token+': %.7f'%eval(parameter_str)

                    replaced_tokens.append((i, parameter_str))
            else:
                replaced_tokens.append((i, token))

        rate_expr = tokenize.untokenize(replaced_tokens)
        try:
            rate_const = eval(rate_expr)
        except Exception as e:
            raise UserWarning(
            "Could not evaluate rate expression: %s\nException: %s" \
                % (rate_expr, e))

    return rate_const

#kmcos can be invoked directly from the command line in one of the following ways::
#    kmcos [help] (all|benchmark|build|edit|export|help|import|rebuild|run|settings-export|shell|version|view|xml) [options]
#defining small wrapper functions to call cli if person types kmcos.export("MyFirstModel.xml") etc. Probably should have made an "@" decorator, but that's okay.
def build(argumentsString):
    import kmcos.cli as cli
    cli.main('build' +' ' + argumentsString)
def edit(argumentsString):
    import kmcos.cli as cli
    cli.main('edit'+' ' + argumentsString)
def export(argumentsString):
    import kmcos.cli as cli
    cli.main('export'+' ' + argumentsString)
def help(argumentsString):
    import kmcos.cli as cli
    cli.main('help' +' '+ argumentsString)
def rebuild(argumentsString):
    import kmcos.cli as cli
    cli.main('rebuild'+' ' + argumentsString)
def run(argumentsString):
    import kmcos.cli as cli
    cli.main('run'+' ' + argumentsString)
def settings_export(argumentsString):
    import kmcos.cli as cli
    cli.main('settings-export'+' ' + argumentsString)
def shell(argumentsString):
    import kmcos.cli as cli
    cli.main('shell'+' ' + argumentsString)    
def version(argumentsString):
    import kmcos.cli as cli
    cli.main('version'+' ' + argumentsString)    
def viewer(argumentsString): #TODO: should change the view.py file to viewer, and change this to view.
    import kmcos.cli as cli
    cli.main('view'+' ' + argumentsString)    
def xml(argumentsString):
    import kmcos.cli as cli
    cli.main('xml'+' ' + argumentsString)    

from kmcos.io import clear_model



# -*- coding: utf-8 -*-
# Copyright: 2009 Nadia Alramli
# License: BSD
"""Draws an animated terminal progress bar
Usage:
    p = ProgressBar("blue")
    p.render(percentage, message)
"""
from __future__ import absolute_import

from . import terminal
import sys

class ProgressBar(object):
    """Terminal progress bar class"""
    TEMPLATE = (
     '%(percent)-2s%% %(color)s%(progress)s%(normal)s%(empty)s %(message)s\n'
    )
    PADDING = 7

    def __init__(self, color=None, width=None, block='█', empty=' '):
        """
        color -- color name (BLUE GREEN CYAN RED MAGENTA YELLOW WHITE BLACK)
        width -- bar width (optinal)
        block -- progress display character (default '█')
        empty -- bar display character (default ' ')
        """
        if color:
            self.color = getattr(terminal, color.upper())
        else:
            self.color = ''
        if width and terminal.COLUMNS and width < terminal.COLUMNS - self.PADDING:
            self.width = width
        else:
            if terminal.COLUMNS :
                # Adjust to the width of the terminal
                self.width = terminal.COLUMNS - self.PADDING
            else:
                self.width = 78
        self.block = block
        self.empty = empty
        self.progress = None
        self.lines = 0

    def render(self, percent, message = ''):
        """Print the progress bar
        percent -- the progress percentage %
        message -- message string (optional)
        """
        inline_msg_len = 0
        if message:
            # The length of the first line in the message
            inline_msg_len = len(message.splitlines()[0])
        if terminal.COLUMNS and inline_msg_len + self.width + self.PADDING > terminal.COLUMNS:
            # The message is too long to fit in one line.
            # Adjust the bar width to fit.
            bar_width = terminal.COLUMNS - inline_msg_len -self.PADDING
        else:
            bar_width = self.width

        # Check if render is called for the first time
        if self.progress != None:
            self.clear()
        self.progress = (bar_width * percent) / 100
        data = self.TEMPLATE % {
            'percent': percent,
            'color': self.color,
            'progress': self.block * int(self.progress),
            'normal': terminal.NORMAL,
            'empty': self.empty * int(bar_width - self.progress),
            'message': message
        }
        sys.stdout.write(data)
        sys.stdout.flush()
        # The number of lines printed
        self.lines = len(data.splitlines())

    def clear(self):
        """Clear all printed lines"""
        sys.stdout.write(
            str(self.lines * (terminal.UP + terminal.BOL + terminal.CLEAR_EOL))
        )



#this file is normally used by "import snapshots_globals as sg"
import kmcos.run
import kmc_settings

simulation_name = kmc_settings.model_name
start = 0.0
model = kmcos.run.KMC_Model()
atoms = model.get_atoms(geometry=False)
occ_header_string = model.get_occupation_header()
occ_header_array = occ_header_string.split()
TOF_header_string = model.get_tof_header()
TOF_header_array = TOF_header_string.split()
data_file_name = simulation_name + str('_TOFs_and_Coverages.csv')
parameters_of_interest = ['T']
steps_so_far = 0
snapshots_so_far = 0
write_output = True
TOF_data_list = []
TOF_integ_list = []
occ_data_list = []
last_snapshot_outputs = []
snapshot_output_headers = []
kmc_time = atoms.kmc_time * 1
config = model._get_configuration().tolist()
PRNG_state = None
steps_before_snapshot = 0.0
sps_actual = 0
tps_actual = 0
snapshots_sampling = None

# List of variables to save. *DO NOT EDIT THIS UNLESS YOU KNOW WHAT YOU ARE
# DOING!*
__var_list__ = ['simulation_name', 'start', 'occ_header_string',
    'occ_header_array', 'TOF_header_string', 'TOF_header_array',
    'data_file_name', 'parameters_of_interest', 'steps_so_far',
    'snapshots_so_far', 'write_output', 'TOF_data_list', 'TOF_integ_list',
    'occ_data_list', 'last_snapshot_outputs', 'snapshot_output_headers',
    'kmc_time', 'config', 'PRNG_state', 'steps_before_snapshot',
    'sps_actual', 'tps_actual', 'snapshots_sampling']



#!/usr/bin/env python
import os

#    Copyright 2009-2013 Max J. Hoffmann (mjhoffmann@gmail.com)
#    This file is part of kmcos.
#
#    kmcos is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    kmcos is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with kmcos.  If not, see <http://www.gnu.org/licenses/>.

APP_ABS_PATH = os.path.dirname(os.path.abspath(__file__))
GLADEFILE = os.path.join(APP_ABS_PATH, 'kmc_editor.glade')
GLADEFILE = 'kmcos/kmc_editor.glade'
try:
    import kiwi
    kiwi.environ.environ.add_resource('glade', APP_ABS_PATH)
except:
    pass



#!/usr/bin/env python3
"""Several utility functions that do not seem to fit somewhere
   else.
"""
#    Copyright 2009-2013 Max J. Hoffmann (mjhoffmann@gmail.com)
#    This file is part of kmcos.
#
#    kmcos is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    kmcos is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with kmcos.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import with_statement
from __future__ import print_function
import re
from time import time
from io import StringIO
from kmcos.utils.ordered_dict import OrderedDict

ValidationError = UserWarning
try:
    from kiwi.datatypes import ValidationError
except:
    print('kiwi Validation not working.')

FCODE = """module kind
implicit none
contains
subroutine real_kind(p, r, kind_value)
  integer, intent(in), optional :: p, r
  integer, intent(out) :: kind_value

  if(present(p).and.present(r)) then
    kind_value = selected_real_kind(p=p, r=r)
  else
    if (present(r)) then
      kind_value = selected_real_kind(r=r)
    else
      if (present(p)) then
        kind_value = selected_real_kind(p)
      endif
    endif
  endif
end subroutine real_kind

subroutine int_kind(p, r, kind_value)
  integer, intent(in), optional :: p, r
  integer, intent(out) :: kind_value

  if(present(p).and.present(r)) then
    kind_value = selected_int_kind(p)
  else
    if (present(r)) then
      kind_value = selected_int_kind(r=r)
    else
      if (present(p)) then
        kind_value = selected_int_kind(p)
      endif
    endif
  endif
end subroutine int_kind

end module kind
"""


class CorrectlyNamed:

    """Syntactic Sugar class for use with kiwi, that makes sure that the name
    field of the class has a name field, that always complys with the rules
    for variables.
    """

    def __init__(self):
        pass

    def on_name__validate(self, _, name):
        """Called by kiwi upon chaning a string
        """
        if ' ' in name:
            return ValidationError('No spaces allowed')
        elif name and not name[0].isalpha():
            return ValidationError('Need to start with a letter')


def write_py(fileobj, images, **kwargs):
    """Write a ASE atoms construction string for `images`
       into `fileobj`.
    """
    import numpy as np

    if isinstance(fileobj, str):
        fileobj = open(fileobj, 'w')

    scaled_positions = kwargs['scaled_positions'] \
        if 'scaled_positions' in kwargs else True
    fileobj.write('from ase import Atoms\n\n')
    fileobj.write('import numpy as np\n\n')

    if not isinstance(images, (list, tuple)):
        images = [images]
    fileobj.write('images = [\n')

    for image in images:
        if hasattr(image, 'get_chemical_formula'):
            chemical_formula = image.get_chemical_formula(mode='reduce')
        else:
            chemical_formula = image.get_name()
        cell_string = repr(image.cell)
        cell_string = cell_string.replace('cell', '')
        cell_string = cell_string.replace('Cell', '')
        fileobj.write("    Atoms(symbols='%s',\n"
                      "          pbc=np.%s,\n"
                      "          cell=np.array(      %s),\n" % (
                          chemical_formula,
                          repr(image.pbc),
                          cell_string))

        if not scaled_positions:
            fileobj.write("          positions=np.array(      %s),\n"
                          % repr(list(image.positions)))
        else:
            fileobj.write("          scaled_positions=np.array(      %s),\n"
                          % repr(list((np.around(image.get_scaled_positions(), decimals=7)).tolist())))
        #print(image.get_scaled_positions())
        fileobj.write('),\n')

    fileobj.write(']')


def get_ase_constructor(atoms):
    """Return the ASE constructor string for `atoms`."""
    if isinstance(atoms, str):
        #return atoms
        atoms = eval(atoms)
    if type(atoms) is type([]):
        atoms = atoms[0]
    f = StringIO()
    write_py(f, atoms)
    f.seek(0)
    lines = f.readlines()
    f.close()
    astr = ''
    for i, line in enumerate(lines):
        if i >= 5 and i < len(lines) - 1:
            astr += line
    # astr = astr[:-2]
    return astr.strip()


def product(*args, **kwds):
    """Take two lists and return iterator producing
    all combinations of tuples between elements
    of the two lists."""
    # product('ABCD', 'xy') --> Ax Ay Bx By Cx Cy Dx Dy
    # product(range(2), repeat=3) --> 000 001 010 011 100 101 110 111
    pools = [tuple(arg) for arg in args] * kwds.get('repeat', 1)
    result = [[]]
    for pool in pools:
        result = [x + [y] for x in result for y in pool]
    for prod in result:
        yield tuple(prod)


def split_sequence(seq, size):
    """Take a list and a number n and return list
       divided into n sublists of roughly equal size.
    """
    newseq = []
    splitsize = 1.0 / size * len(seq)
    for i in range(size):
        newseq.append(seq[int(round(i * splitsize)):
                      int(round((i + 1) * splitsize))])
    return newseq


def download(project):
    from django.http import HttpResponse
    import zipfile
    import tempfile
    from os.path import join, basename
    from glob import glob
    from kmcos.io import import_xml, export_source

    # return HTTP download response (e.g. via django)
    response = HttpResponse(mimetype='application/x-zip-compressed')
    response['Content-Disposition'] = 'attachment; filename="kmcos_export.zip"'

    if isinstance(project, str):
        project = import_xml(project)

    from io import StringIO
    stringio = StringIO()
    zfile = zipfile.ZipFile(stringio, 'w')

    # save XML
    zfile.writestr('project.xml', str(project))

    # generate source
    tempdir = tempfile.mkdtemp()
    srcdir = join(tempdir, 'src')

    # add kMC project sources
    export_source(project, srcdir)
    for srcfile in glob(join(srcdir, '*')):
        zfile.write(srcfile, join('src', basename(srcfile)))

    # add standalone kmcos program
    # TODO

    # write temporary file to response
    zfile.close()
    stringio.flush()
    response.write(stringio.getvalue())
    stringio.close()
    return response


def evaluate_kind_values(infile, outfile):
    """Go through a given file and dynamically
    replace all selected_int/real_kind calls
    with the dynamically evaluated fortran code
    using only code that the function itself
    contains.

    """
    import re
    import os
    import sys
    import shutil
    sys.path.append(os.path.abspath(os.curdir))

    with open(infile) as infh:
        intext = infh.read()
    if not ('selected_int_kind' in intext.lower()
            or 'selected_real_kind' in intext.lower()):
        shutil.copy(infile, outfile)
        return

    def import_selected_kind():
        """Tries to import the module which provides
        processor dependent kind values. If the module
        is not available it is compiled from a here-document
        and imported afterwards.

        Warning: creates both the source file and the
        compiled module in the current directory.

        """
        try:
            import f2py_selected_kind
        except:
            from numpy.f2py import compile
            # quick'n'dirty workaround for windoze
            if os.name == 'nt':
                f = open('f2py_selected_kind.f90', 'w')
                f.write(FCODE)
                f.close()
                from copy import deepcopy
                # save for later
                true_argv = deepcopy(sys.argv)
                sys.argv = (('%s -c --fcompiler=gnu95 --compiler=mingw32'
                             ' -m f2py_selected_kind'
                             ' f2py_selected_kind.f90')
                            % sys.executable).split()
                from numpy import f2py as f2py2e
                f2py2e.main()

                sys.argv = true_argv
            else:
                fcompiler = os.environ.get('F2PY_FCOMPILER', 'gfortran')
                compile(FCODE, source_fn='f2py_selected_kind.f90',
                        modulename='f2py_selected_kind',
                        extra_args='--fcompiler=%s' % fcompiler)
            try:
                import f2py_selected_kind
            except Exception as e:
                raise Exception('Could not create selected_kind module\n'
                                + '%s\n' % os.path.abspath(os.curdir)
                                + '%s\n' % os.listdir('.')
                                + '%s\n' % e)
        return f2py_selected_kind.kind

    def parse_args(args):
        """
            Parse the arguments for selected_(real/int)_kind
            to pass them on to the Fortran module.

        """
        in_args = [x.strip() for x in args.split(',')]
        args = []
        kwargs = {}

        for arg in in_args:
            if '=' in arg:
                symbol, value = arg.split('=')
                kwargs[symbol] = eval(value)
            else:
                args.append(eval(arg))

        return args, kwargs

    def int_kind(args):
        """Python wrapper around Fortran selected_int_kind
        function.
        """
        args, kwargs = parse_args(args)
        return import_selected_kind().int_kind(*args, **kwargs)

    def real_kind(args):
        """Python wrapper around Fortran selected_real_kind
        function.
        """
        args, kwargs = parse_args(args)
        return import_selected_kind().real_kind(*args, **kwargs)

    infile = open(infile)
    outfile = open(outfile, 'w')
    int_pattern = re.compile((r'(?P<before>.*)selected_int_kind'
                              '\((?P<args>.*)\)(?P<after>.*)'),
                             flags=re.IGNORECASE)
    real_pattern = re.compile((r'(?P<before>.*)selected_real_kind'
                               '\((?P<args>.*)\)(?P<after>.*)'),
                              flags=re.IGNORECASE)

    for line in infile:
        real_match = real_pattern.match(line)
        int_match = int_pattern.match(line)
        if int_match:
            match = int_match.groupdict()
            line = '%s%s%s\n' % (
                match['before'],
                int_kind(match['args']),
                match['after'],)
        elif real_match:
            match = real_match.groupdict()
            line = '%s%s%s\n' % (
                match['before'],
                real_kind(match['args']),
                match['after'],)
        outfile.write(line)
    infile.close()
    outfile.close()


def build(options):
    """Build binary with f2py binding from complete
    set of source file in the current directory.

    """

    from os.path import isfile
    import os
    import sys
    from glob import glob

    src_files = ['kind_values_f2py.f90', 'base.f90']
    
    if isfile('base_acf.f90'):
        src_files.append('base_acf.f90')
    src_files.append('lattice.f90')
    if isfile('proclist_constants.f90'):
        src_files.append('proclist_constants.f90')
    if isfile('proclist_pars.f90'):
        src_files.append('proclist_pars.f90')

    src_files.extend(glob('nli_*.f90'))
    # src_files.extend(glob('get_rate_*.f90'))
    src_files.extend(glob('run_proc_*.f90'))
    src_files.append('proclist.f90')
    if isfile('proclist_acf.f90'):
        src_files.append('proclist_acf.f90')

    extra_flags = {}

    if options.no_optimize:
        extra_flags['gfortran'] = ('-ffree-line-length-none -ffree-form'
                                   ' -xf95-cpp-input -Wall -fimplicit-none'
                                   ' -time  -fmax-identifier-length=63 ')
        extra_flags['gnu95'] = extra_flags['gfortran']
        extra_flags['intel'] = '-fpp -Wall -I/opt/intel/fc/10.1.018/lib'
        extra_flags['intelem'] = '-fpp -Wall'

    else:
        extra_flags['gfortran'] = ('-ffree-line-length-none -ffree-form'
                                   ' -xf95-cpp-input -Wall -O3 -fimplicit-none'
                                   ' -time -fmax-identifier-length=63 ')
        extra_flags['gnu95'] = extra_flags['gfortran']
        extra_flags['intel'] = '-fast -fpp -Wall -I/opt/intel/fc/10.1.018/lib'
        extra_flags['intelem'] = '-fast -fpp -Wall'

    # FIXME
    extra_libs = ''
    ccompiler = ''
    if os.name == 'nt':
        ccompiler = '--compiler=mingw64'#'--compiler=mingw32'
        if sys.version_info < (2, 7):
            extra_libs = ' -lmsvcr71 '
        else:
            extra_libs = ' -lmsvcr90 '

    module_name = 'kmc_model'

    if not isfile('kind_values_f2py.f90'):
        evaluate_kind_values('kind_values.f90', 'kind_values_f2py.f90')

    for src_file in src_files:
        if not isfile(src_file):
            raise IOError('File %s not found' % src_file)

    call = []
    call.append('-c')
    call.append('-c')
    call.append('--fcompiler=%s' % options.fcompiler)
    if os.name == 'nt':
        call.append('%s' % ccompiler)
    extra_flags = extra_flags.get(options.fcompiler, '')

    if options.debug:
        extra_flags += ' -DDEBUG'
    #NB presence of " around f90flags argument confuses f2py.  
    #NB Command line argument separation already set by 
    #NB split into separate str items in list. Not
    #NB sure why it ever worked.
    #NB call.append('--f90flags="%s"' % extra_flags)
    call.append('--f90flags=%s' % extra_flags)
    call.append('-m')
    call.append(module_name)
    call += src_files

    print(call)
    from copy import deepcopy
    true_argv = deepcopy(sys.argv)  # save for later
    from numpy import f2py
    sys.argv = call
    try:
        f2py.main()  # Doesn't work according to Alberdi, but works in Erwin's.
    except:
        from subprocess import call
        #'python3' is assumed to be the default command, but it could be 'python'. So we use "sys.executable" to avoid getting the wrong one.
        command = [sys.executable, '-m', 'numpy.f2py', '--fcompiler=' + options.fcompiler, '--f90flags=' + extra_flags, '-m',
                   module_name, '-c'] + src_files
        print(' '.join(command))
        call(command)

            
    sys.argv = true_argv


def T_grid(T_min, T_max, n):
    from numpy import linspace, array
    """Return a list of n temperatures between
       T_min and T_max such that the grid of T^(-1)
       is evenly spaced.
    """

    T_min1 = T_min ** (-1.)
    T_max1 = T_max ** (-1.)

    grid = list(linspace(T_max1, T_min1, n))
    grid.reverse()
    grid = [x ** (-1.) for x in grid]

    return array(grid)


def p_grid(p_min, p_max, n):
    from numpy import logspace, log10
    """Return a list of n pressures between
       p_min and p_max such that the grid of log(p)
       is evenly spaced.
    """
    p_minlog = log10(p_min)
    p_maxlog = log10(p_max)

    grid = logspace(p_minlog, p_maxlog, n)

    return grid


def timeit(func):
    """
    Generic timing decorator

    To stop time for function call f
    just ::
        from kmcos.utils import timeit
        @timeit
        def f():
            ...

     """
    def wrapper(*args, **kwargs):
        time0 = time()
        func(*args, **kwargs)
        print('Executing %s took %.3f s' % (func.__name__, time() - time0))
    return wrapper


def col_tuple2str(tup):
    """Convenience function that turns a HTML type color
    into a tuple of three float between 0 and 1
    """
    r, g, b = tup
    b *= 255
    res = '#'
    res += hex(int(255 * r))[-2:].replace('x', '0')
    res += hex(int(255 * g))[-2:].replace('x', '0')
    res += hex(int(255 * b))[-2:].replace('x', '0')

    return res


def col_str2tuple(hex_string):
    """Convenience function that turns a HTML type color
    into a tuple of three float between 0 and 1
    """
    import gtk
    try:
        color = gtk.gdk.Color(hex_string)
    except ValueError as e:
        raise UserWarning('GTK cannot decipher color string {hex_string}: {e}'.format(**locals()))
    return (color.red_float, color.green_float, color.blue_float)


def jmolcolor_in_hex(i):
    """Return a given jmol color in hexadecimal representation."""
    from ase.data.colors import jmol_colors
    color = [int(x) for x in 255 * jmol_colors[i]]
    r, g, b = color
    a = 255
    color = (r << 24) | (g << 16) | (b << 8) | a
    return color


def evaluate_template(template, escape_python=False, **kwargs):
    """Very simple template evaluation function using only exec and str.format()

    There are two flavors of the template language, depending on wether
    the python parts or the template parts are escaped.

    A template can use the full python syntax. Every line starts with '#@ '
    is interpreted as a template line. Please use proper indentation before
    and note the space after '#@'.

    The template lines are converted to TEMPLATE_LINE.format(locals())
    and thefore every variable in the template line should be escape
    with {}.

    A valid template could be

    for i in range:
        #@ Hello World {i}

    """
    locals().update(kwargs)

    result = ''
    NEWLINE = '\n'
    PREFIX = '#@'
    lines = [line + NEWLINE for line in template.split(NEWLINE)]

    if escape_python:
        # first just replace verbose lines by pass to check syntax
        python_lines = ''
        matched = False
        for line in lines:
            if re.match('^\s*%s ?' % PREFIX, line):
                python_lines += line.lstrip()[3:]
                matched = True
            else:
                python_lines += 'pass # %s' % line.lstrip()
        # if the tempate didn't contain any meta strings
        # just return the original
        if not matched:
            return template
        #NB python3 exec doesn't modify local variables.
        #NB create local dict and copy back "result" explicitly
        #NB if any other local variables are modified, that change
        #NB will be lost.
        ldict = locals().copy()
        exec(python_lines, globals(), ldict)
        result = ldict['result']

        # second turn literary lines into write statements
        python_lines = ''
        for line in lines:
            if re.match('^\s*%s ' % PREFIX, line):
                python_lines += line.lstrip()[3:]
            elif re.match('^\s*%s$' % PREFIX, line):
                python_lines += '%sresult += "\\n"\n' % (
                    ' ' * (len(line) - len(line.lstrip())))
            elif re.match('^$', line):
                # python_lines += 'result += """\n"""\n'
                pass
            else:
                python_lines += '%sresult += ("""%s""".format(**dict(locals())))\n' \
                    % (' ' * (len(line.expandtabs(4)) - len(line.lstrip())),  line.lstrip())

        #NB see note above
        ldict = locals().copy()
        exec(python_lines, globals(), ldict)
        result = ldict['result'] 

    else:
        # first just replace verbose lines by pass to check syntax
        python_lines = ''
        matched = False
        for line in lines:
            if re.match('\s*%s ?' % PREFIX, line):
                python_lines += '%spass %s' \
                    % (' ' * (len(line) - len(line.lstrip())),
                       line.lstrip())

                matched = True
            else:
                python_lines += line
        if not matched:
            return template
        #NB see note above
        ldict = locals().copy()
        exec(python_lines, globals(), ldict)
        result = ldict['result']

        # second turn literary lines into write statements
        python_lines = ''
                                             
        for line in lines:
            if re.match('\s*%s ' % PREFIX, line):
                python_lines += '%sresult += ("""%s""".format(**dict(locals())))\n' \
                    % (' ' * (len(line) - len(line.lstrip())),
                       line.lstrip()[3:])
            elif re.match('\s*%s' % PREFIX, line):
                python_lines += '%sresult += "\\n"\n' % (
                    ' ' * (len(line) - len(line.lstrip())))
            else:
                python_lines += line

        #NB see note above
        ldict = locals().copy()
        exec(python_lines, globals(), ldict) 
        result = ldict['result']

    return result
